/**
 * Ein einfaches Protokoll fr den Spielserver.
 * 
 * @author Bernd Gramlich 
 * @version 2014-12-25
 */
public class Serverprotokoll
{   
    /*
     * Der Dialogzustand des Protokolls.
     */
    private int zustand;

    /*
     * Mein Nick.
     */
    private String nick;

    /*
     * Der Nick meiner Gegnerin.
     */
    private String gegnerin;

    /*
     * Das Spiel, das ich beobachte.
     */
    private String spiel;

    /*
     * Mgliche Werte des Dialogzustands.
     * Der Zustand BEOBACHTEN ist bisher nur skizziert und
     * funktioniert noch nicht.
     */
    private final int NICK_EINGEBEN = 0;
    private final int CHATTEN       = 1;
    private final int SPIELEN       = 2;
    private final int WERTEN        = 3;
    private final int BEOBACHTEN    = 4;

    /**
     * Konstruktor.
     * Zu Dialoganfang muss man den eigenen Nick eingeben.
     */
    public Serverprotokoll ()
    {
        zustand = NICK_EINGEBEN;
    }

    /**
     * Verarbeite die eingabeZeile zu einer Antwort.
     */
    public Antwort verarbeite (String eingabeZeile)
    {
        Antwort rueckgabe = new Antwort ();

        if (eingabeZeile == null)
        {
            rueckgabe.setzeNachrichtAnMich ("Welcome to Connect 4!\n"
            + "Engine (C) 2010 Selina M., Bianca R., Lara R., Sophie T.\n"
            + "Networking (C) 2014 Bernd Gramlich\n"
            + "Enter nick:");
        }
        else
        {
            switch (zustand)
            {
                case NICK_EINGEBEN:
                rueckgabe.setzeBefehl ("nick");
                rueckgabe.setzeArgument (eingabeZeile);
                break;

                case CHATTEN:
                if (eingabeZeile.equals ("commands") || eingabeZeile.equals ("help"))
                {
                    rueckgabe.setzeNachrichtAnMich ("commands | help | ayt | (games) | it acts | nick name | (observe so) | play so | points | points ptn | quit | tell so sth | who | who ptn");
                }
                else if (eingabeZeile.equals ("ayt"))
                {
                    rueckgabe.setzeNachrichtAnMich ("Yes.");
                }
                else if (eingabeZeile.equals ("quit"))
                {
                    rueckgabe.setzeBefehl ("quit");
                    rueckgabe.setzeNachrichtAnMich ("Bye.");
                }
                else if (eingabeZeile.startsWith ("nick "))
                {
                    rueckgabe.setzeBefehl ("nick");
                    rueckgabe.setzeArgument (eingabeZeile.substring (5));
                }
                else if (eingabeZeile.equals ("who"))
                {
                    rueckgabe.setzeBefehl ("who");
                    rueckgabe.setzeArgument ("");
                }
                else if (eingabeZeile.startsWith ("who "))
                {
                    rueckgabe.setzeBefehl ("who");
                    rueckgabe.setzeArgument (eingabeZeile.substring (4));
                }
                else if (eingabeZeile.startsWith ("tell "))
                {
                    String[] bestandteile = eingabeZeile.split (" ");

                    rueckgabe.setzeAdressatin (bestandteile[1]);

                    if (bestandteile.length > 2)
                    {
                        rueckgabe.setzeNachrichtAnAdressatin (nick + " tells you: " + eingabeZeile.substring (bestandteile[1].length () + 6));
                        rueckgabe.setzeNachrichtAnMich ("You tell " + bestandteile[1] + ": " + eingabeZeile.substring (bestandteile[1].length () + 6));
                    }
                    else if (bestandteile.length == 2)
                    {
                        rueckgabe.setzeNachrichtAnAdressatin (nick + " is speechless.");
                        rueckgabe.setzeNachrichtAnMich ("You are speechless.");
                    }
                }
                else if (eingabeZeile.startsWith ("it "))
                {
                    String[] bestandteile = eingabeZeile.split (" ");

                    rueckgabe.setzeNachrichtAnAlle (nick + eingabeZeile.substring (2));
                }
                else if (eingabeZeile.startsWith ("play "))
                {
                    String[] bestandteile = eingabeZeile.split (" ");

                    if (nick.equals (bestandteile[1]))
                    {
                        rueckgabe.setzeNachrichtAnMich ("You are not allowed to play against yourself.");
                    }
                    else
                    {
                        rueckgabe.setzeAdressatin (bestandteile[1]);
                        rueckgabe.setzeNachrichtAnAdressatin (nick + " would like to play against you.");
                        rueckgabe.setzeNachrichtAnMich ("You would like to play against " + bestandteile[1] + ".");
                        rueckgabe.setzeBefehl ("play");
                        rueckgabe.setzeArgument (bestandteile[1]);
                    }

                }
                else if (eingabeZeile.equals ("points"))
                {
                    rueckgabe.setzeBefehl ("points");
                    rueckgabe.setzeArgument ("");
                }
                else if (eingabeZeile.startsWith ("points "))
                {
                    rueckgabe.setzeBefehl ("points");
                    rueckgabe.setzeArgument (eingabeZeile.substring (7));
                }
                else
                {
                    rueckgabe.setzeNachrichtAnAlle (nick + ": " + eingabeZeile);
                }
                break;

                case SPIELEN:
                if (eingabeZeile.equals ("commands") || eingabeZeile.equals ("help"))
                {
                    rueckgabe.setzeNachrichtAnMich ("commands | help | ayt | move clm | resign");
                }
                else if (eingabeZeile.equals ("ayt"))
                {
                    rueckgabe.setzeNachrichtAnMich ("Yes, you are playing against " + gegnerin + ".");
                }
                else if (eingabeZeile.equals ("resign"))
                {
                    rueckgabe.setzeBefehl ("resign");
                }
                else if (eingabeZeile.startsWith ("move "))
                {
                    rueckgabe.setzeBefehl ("move");
                    rueckgabe.setzeArgument (eingabeZeile.substring (5));
                }
                else
                {
                    rueckgabe.setzeAdressatin (gegnerin);
                    rueckgabe.setzeNachrichtAnAdressatin (nick + " tells you: " + eingabeZeile);
                    rueckgabe.setzeNachrichtAnMich ("You tell " + gegnerin + ": " + eingabeZeile);
                }
                break;

                case WERTEN:
                if (eingabeZeile.equals ("commands") || eingabeZeile.equals ("help"))
                {
                    rueckgabe.setzeNachrichtAnMich ("commands | help | ayt | done");
                }
                else if (eingabeZeile.equals ("ayt"))
                {
                    rueckgabe.setzeNachrichtAnMich ("Yes, you have finished your game against " + gegnerin + ".");
                }
                else if (eingabeZeile.equals ("done"))
                {
                    rueckgabe.setzeBefehl ("done");
                }
                else
                {
                    rueckgabe.setzeAdressatin (gegnerin);
                    rueckgabe.setzeNachrichtAnAdressatin (nick + " tells you: " + eingabeZeile);
                    rueckgabe.setzeNachrichtAnMich ("You tell " + gegnerin + ": " + eingabeZeile);
                }
                break;

                case BEOBACHTEN:
                if (eingabeZeile.equals ("commands") || eingabeZeile.equals ("help"))
                {
                    rueckgabe.setzeNachrichtAnMich ("commands | help | ayt | unobserve");
                }
                else if (eingabeZeile.equals ("ayt"))
                {
                    rueckgabe.setzeNachrichtAnMich ("Yes, you are observing the game " + spiel + ".");
                }
                else if (eingabeZeile.equals ("unobserve"))
                {
                    rueckgabe.setzeBefehl ("unobserve");
                    rueckgabe.setzeArgument (spiel);
                }
                else
                {
                    /*
                     * Hier brauchen wir zwei Argumente. Sie mit &&& zu trennen, ist
                     * etwas unsaubere Programmierung.
                     */
                    rueckgabe.setzeBefehl ("comment");
                    rueckgabe.setzeArgument (spiel + "&&&" + eingabeZeile);
                }
                break;
            }
        }

        return rueckgabe;
    }

    /**
     * Chatte.
     */
    public void chatte ()
    {
        zustand = CHATTEN;
    }

    /**
     * Spiele gegen die angegebene Gegnerin.
     */
    public void spiele (String name)
    {
        zustand  = SPIELEN;
        gegnerin = name;
    }

    /**
     * Werte ein beendetes Spiel aus.
     */
    public void werte ()
    {
        zustand = WERTEN;
    }

    /**
     * Beobachte das angegebene Spiel.
     */
    public void beobachte (String name)
    {
        zustand = BEOBACHTEN;
        spiel   = name;
    }

    /**
     * Setze den Nick auf den angegebenen Namen.
     */
    public void setzeNick (String name)
    {
        nick = name;
    }
}